"""
MassGen Display Components

Provides various display interfaces for MassGen coordination visualization.
"""

from .base_display import BaseDisplay
from .terminal_display import TerminalDisplay
from .simple_display import SimpleDisplay
from .rich_terminal_display import (
    RichTerminalDisplay,
    is_rich_available,
    create_rich_display,
)

__all__ = [
    "BaseDisplay",
    "TerminalDisplay",
    "SimpleDisplay",
    "RichTerminalDisplay",
    "is_rich_available",
    "is_textual_available",
    "create_rich_display",
]
